/*********************************************************************
 * Módulo 3. Tutorial de desarrollo con NME. Sprites Animados. 
 * Programación Multimedia y Juegos
 * Autor: David Vallejo Fernández    david.vallejo@tegnix.com
 * Autor: Carlos González Morcillo   carlos.gonzalez@tegnix.com
 *
 * You can redistribute and/or modify this file under the terms of the
 * GNU General Public License ad published by the Free Software
 * Foundation, either version 3 of the License, or (at your option)
 * and later version. See <http://www.gnu.org/licenses/>.
 *
 * This file is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.  
 *********************************************************************/

package;

import util.FpsLabel;

import aze.display.TileLayer;
import aze.display.TileClip;
import aze.display.SparrowTilesheet;
import nme.Assets;
import nme.display.Sprite;
import nme.events.Event;
import nme.events.MouseEvent;
import nme.ui.Keyboard;
import nme.events.KeyboardEvent;
import nme.Lib;

class Ciclo extends Sprite {

  var _layer:TileLayer;       // Capa principal de dibujado
  var _character:TileClip;    // Clip del personaje animado
  var _fpstext:FpsLabel;      // Clase propia para mostrar los FPs

  // Constructor =====================================================
  public function new() {
    super();
    haxe.Timer.delay(createScene, 150);   // Llama a "createScene"
    haxe.Timer.delay(addListeners, 150);  // Llama a "addListeners"
  }

  function createScene():Void {
    var sheetData:String = Assets.getText("assets/atlas.xml");
    var tilesheet:SparrowTilesheet = new SparrowTilesheet
      (Assets.getBitmapData("assets/atlas.png"), sheetData);
    _layer = new TileLayer(tilesheet);

    // Carga del Clip animado (nombre "walk" y a 16 fps)
    _character = new TileClip(_layer, "walk", 16);  
    _character.x = 0;  _character.y = stage.stageHeight / 2.0;
    _layer.addChild(_character);
    addChild(_layer.view);

    // Creación del objeto FpsLabel para mostrar los FPS
    _fpstext = new FpsLabel(20, 20, 0x606060, "assets/cafeta.ttf");  
    addChild(_fpstext);
  }

  function addListeners():Void {
    stage.addEventListener(KeyboardEvent.KEY_DOWN, onKeyPressed);
    stage.addEventListener(MouseEvent.CLICK, onMouseClicked);
    stage.addEventListener(Event.ENTER_FRAME, onEnterFrame);
  }

  // Update ==========================================================
  function goInside(c:TileClip, w:Int, h:Int, d: Float):Bool {
    // Devuelve true si está dentro o camina en dirección al escenario
    if ((c.x > w + c.width / 2) && (d > 0))		return false;
    if ((c.x < - (c.width / 2)) && (d < 0))		return false;
    return true;
  }

  function updateCharacter():Void {
    // Ajuste del incremento en X segun la animacion
    var delta:Float = _character.fps * 0.12;
    if (_character.mirror == 1) delta *= -1;
    if (goInside(_character, stage.stageWidth, stage.stageHeight, delta))
      _character.x += delta;
  }

  function onEnterFrame(event:Event):Void {
    _fpstext.update();  // Actualizamos FPS
    updateCharacter();  // Actualizamos el personaje
    _layer.render();    // Despliegue de la capa principal
  }
  
  // Events ==========================================================
  function onKeyPressed(event:KeyboardEvent):Void {
    switch (event.keyCode) {
    case (Keyboard.A):
      if (_character.fps < 50) _character.fps++;
    case (Keyboard.Z):
      if (_character.fps > 2)  _character.fps--;
    }
    trace ("FPS del personaje: " + _character.fps);
  }

  function onMouseClicked(event:MouseEvent):Void {
    _character.mirror == 1 ? _character.mirror = 0: _character.mirror = 1;
  }

  // Main ============================================================
  static public function main() {
    Lib.current.addChild(new Ciclo());
  }

}
